<?php
require_once("common.php");
require_once("Video.class.php");
require_once("DeleteCityPost.class.php");


/***************************************************

	DeleteCity.class.php
	Sept 17, 2011
	Jeff Crouse - jeff@crouse.cc
	
	This class contains the methods that are used for
	the Wordpress plugin, DeleteCity
	
***************************************************/


class DeleteCity
{
	public $logfile;		// path to the file where we put log messages
	public $runcache;		// path to the runcache 'cron' script
	public $plugin_url;		// The URL of this directory
	public $youtube_dl;		// Path to the youtube-dl executable
	public $fh;				// Handle to the logfile
	public $db_version="1.5";
	public $default_sources;
	
	
	// --------------------------------------------
	function DeleteCity()
	{	
		global $wpdb;
		$wpdb->show_errors();
		$this->plugin_url = WP_PLUGIN_URL.'/'.str_replace(basename(__FILE__),"", plugin_basename(__FILE__));
		$this->runcache = dirname(__FILE__)."/runcache.php";
		$this->logfile = dirname(__FILE__)."/logs/deletecity.log";
		$this->youtube_dl = dirname(__FILE__)."/youtube-dl";
		
		
		// If the existing logfile is too big, rename it.
		// The log function will make a new one at the original name
		if(file_exists($this->logfile) && filesize($this->logfile) > 1048576)
		{
			$newname = dirname(__FILE__)."/logs/deletecity-".date('h-i-s-j-m-y').".log";
			rename($this->logfile, $newname);
		}
		$this->fh = @fopen($this->logfile, 'a');
		
		$this->default_sources =  array(
			"most discussed" => "http://gdata.youtube.com/feeds/api/standardfeeds/most_discussed",
			"top rated" => "http://gdata.youtube.com/feeds/api/standardfeeds/top_rated",
			"most viewed" => "http://gdata.youtube.com/feeds/api/standardfeeds/most_viewed",
			"most recent" => "http://gdata.youtube.com/feeds/api/standardfeeds/most_recent");
	}
	

	// --------------------------------------------
	function init()
	{
		global $_REQUEST;
	
		wp_enqueue_style( 'dc-style', "{$this->plugin_url}styles.css"); 
		
		// Used for the modal dialog on the DeleteCity Status page
		wp_enqueue_style( 'jquery-style', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.1/themes/smoothness/jquery-ui.css'); 
		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-dialog' );
		
		// Used to display videos on the admin side
		wp_enqueue_script( 'jw-player' , "{$this->plugin_url}mediaplayer-5.7/jwplayer.js");
		
		// Handle submitted data
		if(isset($_REQUEST['dc_do_action']) && $_REQUEST['dc_do_action']=='true')
		{
			try {
				switch($_REQUEST['action'])
				{
					case 'stopcache':		$this->stopcache();					break;
					case 'runcache':		$this->runcache();					break;
					case 'post_videos':		$this->post_removed_videos();		break;
					case 'restore_defaults':$this->set_default_options(true);	break;
					case 'save_options':	$this->save_posted_options();		break;
				}
			} catch(Exception $e) {
				$this->log($e->getMessage());
			}
		}
	}	
	
	
	// --------------------------------------------
	function activate() 
	{	
		$this->set_default_options();
		
		// Add the two events to th eschedule
		if (!wp_next_scheduled('runcache_function_hook'))
		{
			$this->log("Adding caching event to schedule");
			$twelve_hours = 60 * 60 * 12;
			wp_schedule_event(time()+$twelve_hours, get_option('dc_cache_schedule'), 'runcache_function_hook' );
		}	
		
		if (!wp_next_scheduled('post_videos_function_hook'))
		{
			$this->log("Adding posting event to schedule");
			$one_day = 60 * 60 * 24 * 7;
			wp_schedule_event(time()+$one_day, get_option('dc_post_schedule'), 'post_videos_function_hook' );
		}
		
		$this->create_tables();
	} 
	
	
	// -------------------------------------------
	// http://codex.wordpress.org/Creating_Tables_with_Plugins
	function create_tables()
	{		
		global $wpdb;
		$sql=$wpdb->prepare("SHOW TABLES LIKE %s", get_option('dc_videos_table'));
		$result = $wpdb->get_results($sql);
		$table_exists = ( count($result) > 0 );
		
  		if(!$table_exists || get_option("dc_db_version")!=$this->db_version)
  		{
			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			$this->log("creating table v{$this->db_version}");
			
			$table = get_option('dc_videos_table');
			$sql="CREATE TABLE {$table} (
			  id int(9) NOT NULL AUTO_INCREMENT,
			  youtube_id varchar(255) DEFAULT NULL,
			  vid_path varchar(255) NOT NULL,
			  attach_id int(9) DEFAULT NULL,
			  title varchar(255) DEFAULT NULL,
			  content text,
			  author varchar(255) DEFAULT NULL,
			  date_added datetime DEFAULT NULL,
			  seen_in_feed datetime DEFAULT NULL,
			  date_posted datetime DEFAULT NULL,
			  feed varchar(255) DEFAULT NULL,
			  status enum('new','cached','removed','expired','posted') DEFAULT 'new',
			  PRIMARY KEY  (id),
			  UNIQUE KEY vid_path (vid_path),
			  UNIQUE KEY youtube_id (youtube_id),
			  UNIQUE KEY attach_id (attach_id)
			);";
			
			dbDelta($sql);
			update_option("dc_db_version", $this->db_version);
		}
	}
	

	// -------------------------------------------
	// Sets default options where none exist
	// ie: will not overwrite existing options (unless $force==true)
	function set_default_options($force=false) 
	{
		global $wpdb;
		$this->log("setting default options");

		$defaults = array(
			'dc_cache_schedule' => 'twicedaily',
			'dc_post_schedule' => 'weekly',
			'dc_cache_dir'=> WP_CONTENT_DIR."/dc_cache",
			'dc_blacklist' => 'amateur, anal, ass, bbw, breast, cock, cumming, cunt, dick, dildo, erotic, facial, femdom, fetish, fisting, fuck, fucking, hardcore, horny, incest, jizz, lesbians, lesbiens, masturbation, milf, nude, orgasm, penis, pokemon, porn, pussy, sex, sexy, shemale, shit, squirt, sybian, tits, tranny, whores, xxx',
			'dc_max_cache_size' => 8000,
			'dc_max_age' => 3,
			'dc_post_title_format' => "DeleteCity: Removed Videos of [date]",
			'dc_rate_limit' => '300k',
			'dc_cat' => 1,
			'dc_share' => 0,
			'dc_videos_table' => "{$wpdb->prefix}deletecity_videos",
			'dc_keywords' => "slow loris, embarassing, bizarre, funny",
			'dc_sources' => $this->default_sources );
		
		foreach($defaults as $option => $value)
		{
			if(!get_option($option))	add_option($option, $value);
			else if($force) 			update_option($option, $value);
		}				
	}
	
	
	// --------------------------------------------
	function deactivate() 
	{
		$this->stopcache();
		
		// Unregister the scheduled events
		if($timestamp = wp_next_scheduled('runcache_function_hook'))
		{
			$this->log("Removing caching event from schedule");
			wp_unschedule_event($timestamp, 'runcache_function_hook' );
		}
		if($timestamp = wp_next_scheduled('post_videos_function_hook'))
		{
			$this->log("Removing posting event from schedule");
			wp_unschedule_event($timestamp, 'post_videos_function_hook' );
		}
	} 
	
	// --------------------------------------------
	function header() 
	{
		//$feed_url = str_replace("rss2", "deletecity", get_bloginfo('rss2_url'));
		$feed_url = get_bloginfo('url')."/?feed=deletecity";
		?>
		<link rel="alternate" type="application/rss+xml" title="DeleteCity RSS Feed" href="<?php echo $feed_url; ?>" />
		<?php
	} 
	
	
	// --------------------------------------------
	function footer()
	{
		?><div id="dc-video-box"></div><?php
	} 
	
	
	// --------------------------------------------
	function warnings()
	{
		$warnings = array();
		if(!is_executable($this->runcache)) 
			$warnings[] = "{$this->runcache} is not executable.";
		if(!is_executable($this->youtube_dl))
			$warnings[] = "{$this->youtube_dl} is not executable";
		if(!file_exists(get_option('dc_cache_dir')))
			mkdir(get_option('dc_cache_dir'), 0777, true);
		if(!is_writable(get_option('dc_cache_dir')))
			$warnings[] = "Your cache directory is not writable.";
		if(!is_writable(dirname($this->logfile)))
			$warnings[] = dirname($this->logfile)." should be writable for logging.";
		$keywords = get_option("dc_keywords");
		if(!$keywords || empty($keywords))
			$warnings[] = "Please provide some keywords on the settings page.";
		if(!is_writable("/tmp"))
			$warnings[] = "Can't write pid file to /tmp.  Please make it writable.";
	
		if(count($warnings)>0)
			include( dirname(__FILE__)."/warnings.php" );
	} 
	
	// --------------------------------------------
	function cron_definer() 
	{
		$schedules['blue_moon'] = array(
			'interval'=> 60*60*24*7*365*99,
			'display'=>  __('Once every 99 years')
		);
		$schedules['weekly'] = array(
			'interval'=> 60*60*24*7,
			'display'=>  __('Once every 7 days')
		);
		$schedules['debug'] = array(
			'interval'=> 60*5,
			'display'=>  __('Every 5 minutes (for testing/debug)')
		);
		return $schedules;
	} 
	
	
	// --------------------------------------------
	function runcache()
	{
		// run the cachiing process in the background.
		$cmd = sprintf("php %s %s %s >> %s 2>&1 &", 
			$this->runcache, $_SERVER['HTTP_HOST'], $_SERVER['SERVER_NAME'], $this->logfile);
		$this->log("calling \"{$cmd}\"");
		
		`$cmd`;
	}
	
	
	// --------------------------------------------------------------------------
	// If the process is still running, it returns the ID.  false otherwise
	function get_runcache_pid()
	{
		// Finds the file created by runcache that contains its PID
		$pid_file = "/tmp/runcache.php.pid";
		if( !file_exists($pid_file) )
		{
			return false;
		}
		$pid = (int)trim( file_get_contents($pid_file) );
		if(!is_numeric($pid))
		{
			return false;
		}
		$process = trim(`ps aux | grep $pid | grep -v grep`);
		if(empty($process))
		{
			unlink($pid_file);
			return false;
		}
		return $pid;
	}
	
	// --------------------------------------------------------------------------
	function stopcache()
	{
		// Kill the runcache process if it is running.
		// It would be great to use the pcntl here, but it's not widely supported
		$pid = $this->get_runcache_pid();
		if(!$pid)
		{
			$this->log("Caching process is not running.");
			return;
		}
		$this->log("attempting to kill pid $pid");
		if(posix_kill($pid, 0)) // see if process is running
		{
			posix_kill($pid, 9);
			$error = posix_get_last_error();
			if($error==0)
			{
				$this->log("Killed process $pid");
				$pid_file = "/tmp/runcache.php.pid";
				unlink($pid_file);
			}
			else
			{
				$this->log("Warning: Couldn't kill caching process ($pid).");
				$this->log(posix_strerror($error));
			}
		}
		else
		{
			$this->log("Caching process ($pid) is not running.");
		}
	}
	
	
	// --------------------------------------------
	function post_removed_videos()
	{
		$videos = Video::get_unposted_removed();
		if(count($videos) > 0)
		{
			$post = new DeleteCityPost();
			$post->post_videos( $videos );
		}
		else
		{
			$this->log("No videos to post");
		}
	}
	
	
	// --------------------------------------------
	function feed()
	{
		/*
		global $wpdb;
		
		$table = get_option('dc_videos_table');
		$sql="SELECT youtube_id FROM `$table` WHERE status='posted' ORDER BY date_added DESC ";
		$result = $wpdb->get_results($sql, ARRAY_A); 
		if($result===false)
			throw new Exception("Problem fetching unposted_removed");
		$num_items = count($result);
		*/
		include(dirname(__FILE__).'/feed.php');
	}
	
	
	
	// ---------------------------------------
	// Edits the attachment page for posted videos
	// NOTE:  We don't want to use the database in this function because we want the 
	// existing entries to stay intact even if the database gets corrupted or something
	function the_content_filter( $content )
	{
		global $post;
		
		// Get some meta-info added to the attachment by Video::attach_to_post()
		$youtube_id = get_post_meta($post->ID, 'youtube_id', true);
		$author = get_post_meta($post->ID, 'youtube_author', true);
		
		// If it's a DeleteCity attachemnt we are dealing with, insert the jwplayer
		if(is_attachment() && $youtube_id)
		{
			$vid_url = wp_get_attachment_url( $post->ID );
			$thumb_url = str_replace("mp4", "jpg", $vid_url);
			ob_start();
			?>
			<p>by <a href="http://www.youtube.com/user/<?php echo $author; ?>" target="_blank"><?php echo $author; ?></a></p>
			<div id="video-<?php echo $youtube_id; ?>">Loading the player ...</div>
			[<a href="<?php echo $vid_url; ?>">download</a>] 
			[<a href="http://www.youtube.com/watch?v=<?php echo $youtube_id; ?>" target="_blank">original URL</a>]

			<script type="text/javascript"> 
			jwplayer("video-<?php echo $youtube_id; ?>").setup({
				image: "<?php echo $thumb_url; ?>",
				flashplayer: "<?php echo $this->plugin_url; ?>mediaplayer-5.7/player.swf", 
				file: "<?php echo $vid_url; ?>", 
				width: 640,
				height: 360 
			});
			</script>
			<?php
			$content = ob_get_clean();
		}
		return $content;
	}
	
	
	// --------------------------------------------
	function add_admin_menus()
	{
		add_options_page('Delete City Settings', 'Delete City Settings', 'administrator', 'deletecity', array($this, 'settings_page'));
		add_plugins_page("Delete City Status", "Delete City Status", 'administrator', 'deletecity-status', array($this, 'status_page') );
	}
	
	
	// --------------------------------------------
	function status_page()
	{
		$pid = $this->get_runcache_pid();
		$logfile_url = "{$this->plugin_url}".str_replace(dirname(__FILE__), "", $this->logfile);
		include( dirname(__FILE__)."/status.php" );
	}
	
	
	// --------------------------------------------
	function settings_page()
	{
		//must check that the user has the required capability 
		if (!current_user_can('manage_options'))
			wp_die( __('You do not have sufficient permissions to access this page.') ); 
	
		$pid = $this->get_runcache_pid();
		$schedules = wp_get_schedules();
		$sources = get_option('dc_sources');

		$categories = get_categories(array('orderby'=>'name','order'=>'ASC','hide_empty'=>false));
		
		include( dirname(__FILE__)."/settings.php" );
	}
	
	
	// --------------------------------------------
	// AJAX function - Used on the DeleteCity Status page when user clicks on a video
	function load_video_player()
	{
		$video = Video::loadFromYouTubeID($_POST['youtube_id']);
		?>
			<div class="video-title" style="font-size: 18px; font-weight: bold;"><?php echo trunc($video->title,50); ?></div>
			<p>by <a href="http://www.youtube.com/user/<?php echo $video->author; ?>" target="_blank"><?php echo $video->author; ?></a></p>
			<div id="video-<?php echo $youtube_id; ?>">Loading the player ...</div>
			[<a href="<?php echo $video->vid_url; ?>">download</a>] 
			[<a href="http://www.youtube.com/watch?v=<?php echo $video->youtube_id; ?>" target="_blank">original URL</a>]
			<p><?php echo $video->content; ?></p>
			<script type="text/javascript"> 
			jwplayer("video-<?php echo $youtube_id; ?>").setup({
				image: "<?php echo $video->thumb_url; ?>",
				flashplayer: "<?php echo $this->plugin_url; ?>mediaplayer-5.7/player.swf", 
				file: "<?php echo $video->vid_url; ?>", 
				width: 640,
				height: 360 
			});
			</script>
		<?php
		die();
	}
	
	// --------------------------------------------
	// AJAX function - used on the DeleteCity Status page
	function get_vids()
	{
		global $wpdb;
	
		if(isset($_POST['dc_delete']))
		{
			try {
				Video::delete( $_POST['dc_delete'] );
			} catch(Exception $e) {
				print "Couldn't delete '{$_POST['dc_delete']}'.  ".$e->getMessage();
			}
		}
		
		$page = intval($_POST['dc_page']);
		$filter = $_POST['dc_filter'];
		$results_per_page = 12;
		$offset = $results_per_page * $page;
	
		// Assemble the SQL 
		$table = get_option('dc_videos_table');
		$sql="SELECT youtube_id FROM `$table`";
		
		if($filter=='cached')		$sql .= " WHERE status='cached' ";
		if($filter=='posted')		$sql .= " WHERE status='posted' ";
		if($filter=='removed')		$sql .= " WHERE status='removed' ";
		if($filter=='expired')		$sql .= " WHERE status='expired' ";
		if(strpos($filter, "http://gdata.youtube.com/feeds")!==false)	$sql .= " WHERE feed='$filter' ";
		$sql .= " ORDER BY date_added DESC ";
		
		// Get the total number of videos
		$results = $wpdb->get_results($sql);
		if($result===false) $wpdb->print_error();
			
		$total = count( $results );
	
		// Get the videos for this page
		$sql .= " LIMIT $results_per_page OFFSET $offset";
		$videos = $wpdb->get_results($sql, ARRAY_A); 

		$ar = getDirectorySize(get_option('dc_cache_dir'), "mp4"); 
		
		include( dirname(__FILE__)."/admin_vids.php" );
		die();
	}
	

	// --------------------------------------------
	// AJAX function - used on the DeleteCity Status page
	function get_status()
	{
		$time = wp_next_scheduled('runcache_function_hook')-time();
		$seconds = $time % 60;
		$mins = floor($time/60) % 60;
		$hours = floor($time/60/60) % 24;
		$days = floor($time/60/60/24);
		
		$parts = array();
		if($days) $parts[] = 	"{$days} days";
		if($hours) $parts[] = 	"{$hours} hours";
		if($mins) $parts[] = 	"{$mins} minutes";
		if($seconds) $parts[] = "{$seconds} seconds";
		$interval = implode(", ", $parts);


		$response=array("buttons"=>"", "status"=>"", "log"=>"");
		//$output = `ps aux | grep runcache | grep -v grep`;
		
		$response['pid'] = $this->get_runcache_pid();
		
		ob_start();
		if($this->get_runcache_pid()): ?>
			<img src="<?php echo $this->plugin_url; ?>gs/ajax-loader.gif" /> <a title="<?php echo  `ps aux | grep runcache | grep -v grep`; ?>">The cache process is running</a> 
			<?php 
				if(get_option('dc_current_feed_number')!==false)
					printf("Feed %d/%d, Entry %d/%d", 
						get_option('dc_current_feed_number')+1, 
						get_option('dc_total_feeds'), 
						get_option('dc_current_entry_number')+1, 
						get_option('dc_total_entries') );
			?>
		<?php else: ?>
			The caching process is not running.  It will run in <?php echo $interval; ?>
		<?php endif;
		$response['status'] = ob_get_clean();
		
		$response['log'] = "NOTE:  This log is truncated.  Click the arrow on the upper left to see the entire file.\n\n";
		$response['log'] .= last_lines($this->logfile, 20);
		print json_encode($response);
		die();
	}
	

	// --------------------------------------------
	function log($message) 
	{
		fwrite($this->fh, "[deletecity] ".date(DATE_ATOM)." $message\n");
	}
	
	
	// --------------------------------------------
	// Uses the $_REQUEST global array to set all of the Wordpress options
	// Called when the DeleteCity Settings page is submitted
	// TO DO:  validate $_REQUEST stuff
	function save_posted_options()
	{
		global $_REQUEST;
		$this->deactivate();
		
		update_option('dc_sources', $_REQUEST['dc-sources']);
		update_option('dc_keywords', $_REQUEST['dc-keywords']);
		update_option('dc_cache_schedule', $_REQUEST['dc-cache-schedule']);
		update_option('dc_post_schedule',  $_REQUEST['dc-post-schedule']);
		update_option('dc_max_cache_size',  $_REQUEST['dc-max-cache-size']);
		update_option('dc_rate_limit',  $_REQUEST['dc-rate-limit']);
		if($_REQUEST['dc-max-age'] > 1 && $_REQUEST['dc-max-age'] < 10)
			update_option('dc_max_age', intval($_REQUEST['dc-max-age']));
		update_option('dc_cat', $_REQUEST['dc-category']);
		update_option('dc_share', $_REQUEST['dc-share']);
		update_option('dc_post_title_format', $_REQUEST['dc-post-title-format']);
		
		$words = array();		
		foreach(preg_split("/[\s]*[,][\s]*|[\n]/", $_REQUEST['dc-blacklist']) as $word)
		{
			$word = trim($word);
			$word = strtolower($word);
			if(!empty($word))
				$words[] = $word;
		}
		
		$blacklist = array_unique($words);
		sort($words);
		update_option('dc_blacklist',  $words);
		
		$this->activate();
	}  
}
?>