<?php

/***************************************************

	DeleteCityPost.class.php
	Sept 17, 2011
	Jeff Crouse - jeff@crouse.cc
	
	This class represents a single post on a 
	Wordpress blog.  It can contain multiple videos
	
***************************************************/

class DeleteCityPost 
{

	public $post_id;
	public $logfile;		// path to the file where we put log messages
	
	
	// --------------------------------------------
	function DeleteCityPost()
	{
		$this->logfile = dirname(__FILE__)."/logs/deletecity.log";
		$this->fh = @fopen($this->logfile, 'a');
	}


	// --------------------------------------------
	// 5 steps!  
	function post_videos($videos)
	{
		// STEP 1.  Create a placeholder post
		$this->post_id = DeleteCityPost::create_draft();
		$this->log("Created draft for videos with ID={$this->post_id}");
		
		// STEP 2. Add each video as an attachment to the placeholder post
		foreach($videos as $video)
		{	
			$attach_id = $video->attach_to_post( $this->post_id );
			$this->log("Attached {$video->youtube_id} to post with attach_id={$attach_id}");
		}
	
		// STEP 3. Update the post into the database
		$this->insert_attachments_into_post();
		
		// STEP 4. Publish the post! 
		$this->publish();
		
		// STEP 5. Mark all of the videos as posted
		foreach($videos as $video)
		{
			$this->log("Marking {$video->youtube_id} as posted");
			$video->mark_as_posted();
		}
	}


	
	// --------------------------------------------
	// Finds all of the attachments that were made in STEP 2 of post_videos
	// and inserts an HTML representation of them into the draft post made in STEP 1
	function insert_attachments_into_post()
	{
		// Find the post we want
		$p = new WP_Query("p={$this->post_id}");
		$p->the_post();

		// Assembly the post content
		$more = $num_videos-6;
		$i=0;
		$content = '<div class="dc-videos">';
		
		$args = array('post_type' => 'attachment', 'numberposts' => -1, 'post_status' => null, 'post_parent' => $this->post_id); 
		$attachments = get_posts($args);
		if(!$attachments)
			throw new Exception("No attachments to post?");
		//$this->log( var_export($attachments,true) );
		
		$num_attachments = count($attachments);
		$this->log("{$num_attachments} attachments found");
		foreach($attachments as $attachment)
		{
			$this->log("Generating HTML for attachment {$attachment->ID}");
			$link = get_attachment_link($attachment->ID);
			$youtube_id = get_post_meta($attachment->ID, 'youtube_id', true);
			$video = Video::loadFromYouTubeID($youtube_id);
			
			$html = <<<EOT
			<div class="dc-video">
				<div class="dc-title">{$video->title}</div>
				<a href="{$link}"><img src="{$video->thumb_url}" class="dc-thumbnail" /></a>
				<div class="dc-author">
					<b>by:</b><a href="http://www.youtube.com/user/{$video->author}" target="_blank">{$video->author}</a>
				</div>
			</div>
EOT;
			$content .= $html;
			if($i==6) $content .= "<!--more {$more} more videos after the break. -->";
			$i++;
		}

		wp_reset_postdata();
		$content .= '</div>';
		
		$result = wp_update_post( array("ID"=>$this->post_id, "post_content"=>$content) );	
		if($result==0)
			throw new Exception("Couldn't update post {$this->post_id} with new content. Now everything is fucked.");
	}
	
	
	// --------------------------------------------
	function publish()
	{
		$this->log("Publishing post {$this->post_id}");
		$post = array("ID"=>$this->post_id, "post_status"=>'publish');
		$result = wp_update_post( $post );	
		if($result==0)
			throw new Exception("Couldn't publish {$this->post_id}.");
		
	}


	// --------------------------------------------
	static function create_draft()
	{
		$title = get_option('dc_post_title_format');
		$title = str_replace("[date]", date("F j, Y, g:i a"), $title);
		$my_post = array(
			'post_title' => $title,
			'post_content' => "placeholder",
			'post_status' => 'draft',
			'post_author' => 1,
			'post_category' => array(get_option('dc_cat'))
		);
		$post_id = wp_insert_post( $my_post );
		if($post_id==0)
			throw new Exception("Couldn't create placeholder post");
		if(is_wp_error( $post_id ))
			throw new Exception(implode("\n", $post_id->errors()));
		
		return $post_id;
	}
	
	// --------------------------------------------
	function log($message) 
	{
		fwrite($this->fh, "[deletecity] ".date(DATE_ATOM)." $message\n");
	}

}
?>