<?php
require_once("common.php");


/***************************************************

	Video.class.php
	Sept 17, 2011
	Jeff Crouse - jeff@crouse.cc
	
	This class represents a single video from YoTube
	
***************************************************/

class Video {
	
	// database fields
	var $id=NULL;
	var $title=NULL;
	var $content=NULL;
	var $author=NULL;
	var $status='new';
	var $youtube_id=NULL;
	var $date_added;		// in PHP, it is unixtime.  in database, DATETIME
	var $seen_in_feed;		// This gets updated any time a video appears one of the sources. It's "age" is calculated based on this rather than date_added
	var $date_posted=NULL;	// in PHP, it is unixtime.  in database, DATETIME
	var $feed=NULL;
	var $attach_id=NULL;
	var $vid_path=NULL;
	
	// other
	var $age;		// Age (in days = NOW()-seen_in_feed)
	var $thumb_path;
	var $data_path;
	var $in_db=false;


	// ----------------------------------------------
	function Video()
	{
		global $wpdb;
		$wpdb->show_errors();
		$this->age = 0;
	}
	
	
	// ----------------------------------------------
	static function get_duration($path)
	{
		ob_start();
		passthru("ffmpeg -i \"{$path}\" 2>&1");
		$duration = ob_get_contents();
		ob_end_clean();
		
		$search='/Duration: (.*?)\./';
		if(preg_match($search, $duration, $matches, PREG_OFFSET_CAPTURE, 3))
		{
			return $matches[1][0];
		}
	}
	
	
	// ----------------------------------------------
	function get_filesize()
	{
		return intval( @filesize($this->vid_path) );
	}
	
	
	// ----------------------------------------------
	function expire()
	{
		global $wpdb;
		@unlink($this->vid_path);
		
		if(!file_exists($this->vid_path))
		{
			$result = $wpdb->update(get_option('dc_videos_table'), 
				array('status' => 'expired'), 
				array('youtube_id' => $this->youtube_id)
			);
			if(!$result)
				throw new Exception("Problem expiring $this->youtube_id");
		}
		else
			throw new Exception("Could not delete {$this->youtube_id}.  Check permissions.");
	}
	
	
	// ----------------------------------------------
	function seen_in_feed()
	{
		global $wpdb;
		$this->seen_in_feed=time();
		$table = get_option('dc_videos_table');
		$sql = $wpdb->prepare("UPDATE `$table` SET `seen_in_feed`=FROM_UNIXTIME(%d) WHERE youtube_id=%s", 
			$this->seen_in_feed,
			$this->youtube_id);
		
		$result = $wpdb->query($sql);
		if($result===false) strip_tags($wpdb->print_error());
	}
	
	
	// ----------------------------------------------
	function move_to_uploads()
	{
		global $wpdb;
		$uploads = wp_upload_dir();
		
		// Is it already in the uploads directory?
		if(strpos($this->vid_path, $uploads['basedir'])!==false) return;
		
		// Create the new name and move the file
		$new_path = sprintf("%s/%s.mp4", $uploads['path'], $this->youtube_id);
		rename($this->vid_path, $new_path);
		
		// Check if the move was successful.
		if(!file_exists($new_path))
			throw new Exception("Couldn't move {$this->vid_path} to {$new_path}");
		
		// Update the database
		$result = $wpdb->update(get_option('dc_videos_table'), 
			array('vid_path' => $new_path), 
			array('youtube_id' => $this->youtube_id)
		);
		if(!$result)
			throw new Exception("Problem updating vid_path for $this->youtube_id");
		
		// Update the local vars 
		$this->vid_path = $new_path;
		
		// Move the thumbnail to the new location
		$new_thumb_path = str_replace("mp4", "jpg", $this->vid_path);
		rename($this->thumb_path, $new_thumb_path);
		$this->thumb_path = $new_thumb_path;
		
		// Move the data file to the new location
		$new_data_path = str_replace("mp4", "xml", $this->vid_path);
		rename($this->data_path, $new_data_path);
		$this->data_path = $new_data_path;

		if(function_exists('content_url'))
		{
			$wp_content_dir = ABSPATH . 'wp-content';
			$this->thumb_url = content_url() . str_replace($wp_content_dir, "", $this->thumb_path);
			$this->vid_url = content_url() . str_replace($wp_content_dir, "", $this->vid_path);
		}
	}
	
	
	// ----------------------------------------------
	function attach_to_post($post_id)
	{
		global $wpdb;
	
		// If the video isn't already in the wp-uploads directory, move it there
		$this->move_to_uploads();
		
		// Has it already been attached?
		if($this->attach_id) return $this->attach_id;
		
		$wp_filetype = wp_check_filetype(basename($this->vid_path), null);
		$attachment = array(
			'post_mime_type' => $wp_filetype['type'],
			'post_title' => $this->title, //preg_replace('/\.[^.]+$/', '', basename($filename)),
			'post_content' => "",
			'post_status' => 'inherit'
		);
		$attach_id = wp_insert_attachment( $attachment, $this->vid_path, $post_id );
		if($attach_id==0)
			throw new Exception("Couldn't create attachment for {$video->youtube_id}");

		require_once(ABSPATH . 'wp-admin/includes/image.php');
		$attach_data = wp_generate_attachment_metadata( $attach_id, $this->vid_path );
		$result = wp_update_attachment_metadata( $attach_id, $attach_data );
		if(!$result)
			throw new Exception("wp_update_attachment_metadata failed for {$video->youtube_id}");
		
		// Update the attach_id in the database
		$result = $wpdb->update(get_option('dc_videos_table'), 
			array('attach_id' => $attach_id ), 
			array('youtube_id' => $this->youtube_id)
		);
		if(!$result)
			throw new Exception("Problem updating attach_id for $this->youtube_id");
		$this->attach_id = $attach_id;
		

		add_post_meta($attach_id, "youtube_id", $this->youtube_id);
		add_post_meta($attach_id, "youtube_author", $this->author);
		
		return $this->attach_id;
	}
	
	
	// ----------------------------------------------
	function mark_as_removed()
	{
		global $wpdb;
		$result = $wpdb->update(get_option('dc_videos_table'), 
			array('status' => 'removed'), 
			array('youtube_id' => $this->youtube_id)
		);
		if(!$result)
			throw new Exception("Problem marking as removed for $this->youtube_id");
		$this->status='removed';
	}
	
	
	// ----------------------------------------------
	function mark_as_posted()
	{
		global $wpdb;
		$this->date_posted = time();
		$table = get_option('dc_videos_table');
		$sql = $wpdb->prepare("UPDATE `$table` SET `status`='posted', `date_posted`=FROM_UNIXTIME(%d) WHERE youtube_id=%s", 
			$this->date_posted,
			$this->youtube_id);

		$result = $wpdb->query($sql);
		if($result===false) strip_tags($wpdb->print_error());
	}
	
	
	// ----------------------------------------------
	static function get_unposted_removed()
	{
		global $wpdb;
		
		$table = get_option('dc_videos_table');
		$sql = "SELECT youtube_id FROM $table WHERE status='removed' AND date_posted IS NULL";
		$result = $wpdb->get_results($sql, ARRAY_A); 
		if($result===false)
			throw new Exception("Problem fetching unposted_removed");

		$videos = array();
		foreach($result as $row)
		{
			$videos[] = Video::loadFromYouTubeID( $row['youtube_id'] );
		}
		return $videos;
	}
	
	// ----------------------------------------------
	static function delete($youtube_id)
	{
		global $wpdb;
		$video = Video::loadFromYouTubeID( $youtube_id );

		@unlink($video->vid_path);
		@unlink($video->thumb_path);
		@unlink($video->data_path);
		
		if(!file_exists($video->vid_path) && !file_exists($video->thumb_path))
		{
			$table = get_option('dc_videos_table');
			$sql=$wpdb->prepare("DELETE FROM $table WHERE youtube_id=%s LIMIT 1", $youtube_id);
			$result = $wpdb->query($sql);
			if($result===false)
				throw new Exception("Problem deleting $youtube_id from database");
		}
		else
			throw new Exception("Could not delete {$youtube_id}.  Check permissions.");
	}

	
		

	
	// ------------------------------------
	static function get_web_page( $url, $get_body=true )
	{
		$ch      = curl_init( $url );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_HEADER, false );
		curl_setopt( $ch, CURLOPT_ENCODING, "" );
		curl_setopt( $ch, CURLOPT_USERAGENT, "spider" );
		curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 120 );
		curl_setopt( $ch, CURLOPT_TIMEOUT, 120 );
		curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
		curl_setopt( $ch, CURLOPT_NOBODY, $get_body);
		$content = curl_exec( $ch );
		$err     = curl_errno( $ch );
		$errmsg  = curl_error( $ch );
		$header  = curl_getinfo( $ch );
		curl_close( $ch );
	
		$header['errno']   = $err;
		$header['errmsg']  = $errmsg;
		$header['content'] = $content;
		return $header;
	}
	
	
	
	// ------------------------------------
	static function get_all_feeds()
	{
		global $wpdb;
		$table = get_option('dc_videos_table');
		$sql="SELECT DISTINCT feed FROM `$table`";
		$result = $wpdb->get_results($sql, ARRAY_A); 
		if($result===false)
			throw new Exception("Problem fetching feeds");

		$feeds = array();
		foreach($result as $row)
		{
			$feeds[] = $row['feed'];
		}
		return $feeds;
	}
	
	// ------------------------------------
	static function loadFromYouTubeID($youtube_id)
	{
		global $wpdb;	
		$video = new Video();
		$video->youtube_id = $youtube_id;

		$table = get_option('dc_videos_table');
		$sql = $wpdb->prepare("SELECT *,
			UNIX_TIMESTAMP(date_added) AS date_added,
			UNIX_TIMESTAMP(seen_in_feed) AS seen_in_feed
			FROM `$table` WHERE youtube_id=%s LIMIT 1", $youtube_id);
		$row = $wpdb->get_row($sql, ARRAY_A);
		
		if($row==NULL) return false;

		$video->id = 			$row['id'];
		$video->attach_id = 	$row['attach_id'];
		$video->title = 		$row['title'];
		$video->content = 		$row['content'];
		$video->author = 		$row['author'];
		$video->date_added = 	$row['date_added'];
		$video->seen_in_feed = 	$row['seen_in_feed'];
		$video->status = 		$row['status'];
		$video->vid_path = 		$row['vid_path'];
		$video->date_posted = 	$row['date_posted'];
		$video->in_db =			true;
		$video->age = 			time()-$video->seen_in_feed;
		
		$video->thumb_path = str_replace("mp4", "jpg", $video->vid_path);
		$video->data_path = str_replace("mp4", "xml", $video->vid_path);
		
		if(function_exists('content_url'))
		{
			$wp_content_dir = ABSPATH . 'wp-content';
			$video->thumb_url = content_url() . str_replace($wp_content_dir, "", $video->thumb_path);
			$video->vid_url = content_url() . str_replace($wp_content_dir, "", $video->vid_path);
		}
		return $video;
	}

	
	
	// ----------------------------------------------
	// Use the HTTP code response from fetching the data feed for this video
	// to determine if it still exists.  
	function remote_still_exists()
	{
		$url = "http://gdata.youtube.com/feeds/api/videos/{$this->youtube_id}";
		$request = Video::get_web_page( $url, false );
		
		switch($request['http_code'])
		{
			case 404:	// The only code that means that it has been removed, I think...
				return false;

			case 402:	// Means that YouTube id throttling you.  Useless to do anything after this
				throw new Exception("Youtube has blocked you. Stopping now to cool off.");

			case 403:	// = "Forbidden", but the video is still there...  Why?
			case 200:	// OK
				// TO DO: We might want to double-check whether the video is there.
				// I imagine YouTube could, one day, keep the feed there even after video is removed
				return true;

			default:	// Even if we get an unknown code, assume it is still there.	
				print "\n\n[ALERT] Received unknown HTTP code: {$request['http_code']} for {$this->youtube_id}\n\n";
				return true;
		}			
	}
	
	
	// ------------------------------------
	static function create($vid_url, $title, $content, $author, $feed="")
	{
		global $wpdb;
		
		preg_match("/v=([^&]+)/", $vid_url, $matches);
		if( count($matches) < 2 )
			throw new Exception("Couldn't find 'v=' in {$vid_url}");
		
		if(Video::exists($matches[1]))
			throw new Exception("Trying to create a video that already exists");
		
		$video = new Video();
		$video->youtube_id = $matches[1];
		$video->vid_path = get_option('dc_cache_dir')."/{$video->youtube_id}.mp4";
		$video->thumb_path = str_replace("mp4", "jpg", $video->vid_path);
		$video->data_path = str_replace("mp4", "xml", $video->vid_path);
		$video->title = $title;
		$video->content = $content;
		$video->author = $author;
		$video->feed = $feed;
		$video->date_added = time();
		$video->seen_in_feed = time();
		
		if(!file_exists($video->vid_path))
		{
			$cmd = sprintf('%s/youtube-dl --quiet --continue --no-overwrites --ignore-errors --format=18 --output="%s/%s" --rate-limit=%s %s > /dev/null 2>&1', 
				dirname(__FILE__), 
				get_option('dc_cache_dir'),
				"%(id)s.%(ext)s",
				get_option('dc_rate_limit'),
				$vid_url);
			print "\t\t\tDownloading {$video->youtube_id}, \"{$video->title}\"\n";
			`$cmd`;
		}
		
		if(!file_exists($video->vid_path))
			return NULL;
		
		$table = get_option('dc_videos_table');
		$sql = $wpdb->prepare("INSERT INTO `$table` (youtube_id, vid_path, title, content, author, feed, status, date_added, seen_in_feed) 
			VALUES (%s, %s, %s, %s, %s, %s, 'cached', FROM_UNIXTIME(%d), FROM_UNIXTIME(%d))",
			$video->youtube_id,
			$video->vid_path,
			$video->title,
			$video->content,
			$video->author,
			$video->feed,
			$video->date_added,
			$video->seen_in_feed);	
		$result = $wpdb->query($sql);
		if($result===false) {
			throw new Exception( strip_tags($wpdb->print_error()) );
		} else {
			$video->in_db = true;
			$video->id = $wpdb->insert_id;
		}
		
		$url = "http://img.youtube.com/vi/{$video->youtube_id}/0.jpg";	
		Video::download($url, $video->thumb_path);
		
		$url = "http://gdata.youtube.com/feeds/api/videos/{$video->youtube_id}";	
		Video::download($url, $video->data_path);
		
		return $video;
	}
	
	
	// ------------------------------------
	static function exists($youtube_id)
	{
		global $wpdb;
		$table = get_option('dc_videos_table');
		$sql = $wpdb->prepare( "SELECT id FROM `$table` WHERE youtube_id=%s", $youtube_id);
		$result = $wpdb->get_row($sql);
		return $result != NULL;
	}
	
	
	// ------------------------------------
	static function download($url, $file)
	{
		if(ini_get('allow_url_fopen')=="1")
		{
			@file_put_contents($file, @file_get_contents($url));
			return true;
		}
		
		if(function_exists("curl_init"))
		{
			$ch = curl_init($url);
			$fp = fopen($file, 'wb');
			curl_setopt($ch, CURLOPT_FILE, $fp);
			curl_setopt($ch, CURLOPT_HEADER, 0);
			curl_exec($ch);
			curl_close($ch);
			if(curl_errno($ch))
				throw new Exception(curl_error($ch));
			fclose($fp);
			return true;
		}
		
		return false;
	}
}
?>