<?php
date_default_timezone_set('America/New_York');




// -----------------------------------------------------
function get_wp_load_path()
{
    $base = dirname(__FILE__);
    $path = false;

    if (@file_exists(dirname(dirname($base))."/wp-load.php"))
    {
        $path = dirname(dirname($base))."/wp-load.php";
    }
    else
    if (@file_exists(dirname(dirname(dirname($base)))."/wp-load.php"))
    {
        $path = dirname(dirname(dirname($base)))."/wp-load.php";
    }
    else
    $path = false;

    if ($path != false)
    {
        $path = str_replace("\\", "/", $path);
    }
    return $path;
}



// -----------------------------------------------------
function display_xml_error($error, $xml)
{
    $return  = $xml[$error->line - 1] . "\n";
    $return .= str_repeat('-', $error->column) . "^\n";

    switch ($error->level)
    {
        case LIBXML_ERR_WARNING:
            $return .= "Warning $error->code: ";
            break;
         case LIBXML_ERR_ERROR:
            $return .= "Error $error->code: ";
            break;
        case LIBXML_ERR_FATAL:
            $return .= "Fatal Error $error->code: ";
            break;
    }

    $return .= trim($error->message) .
               "\n  Line: $error->line" .
               "\n  Column: $error->column";

    if ($error->file)
    {
        $return .= "\n  File: $error->file";
    }

    return "$return\n\n--------------------------------------------\n\n";
}



// -----------------------------------------------------
function trunc($string, $limit, $break=" ", $pad="...")
{
  // return with no change if string is shorter than $limit
  if(strlen($string) <= $limit) return $string;

  $string = substr($string, 0, $limit);
  if(false !== ($breakpoint = strrpos($string, $break))) {
    $string = substr($string, 0, $breakpoint);
  }

  return $string . $pad;
}




// -----------------------------------------------------
function last_lines($filename, $lines)
{
        $handle = @fopen($filename, "r");
        if (!$handle) {
            echo "Error: can't find or open $filename<br/>\n";
            return -1;
        }
       $linecounter = $lines;
       $pos = -1;
       $beginning = false;
       $text = array();
       
       /*read until more lines are needed*/
       while ($linecounter > 0) {
               $t = " ";

               /*read until you have found a newline character*/
               while ($t != "\n") {
                       /* if fseek() returns -1 we need to break the cycle*/
                       if(fseek($handle, $pos, SEEK_END) == -1) {
                               $beginning = true; 
                               break;
                       }
                       $t = fgetc($handle);
                       $pos --;
                }

                /*decrement the number of lines we still have to read*/
                $linecounter --;

                if($beginning) {
                    rewind($handle);
                }

                /*read an entire line from the current position*/        
                $text[$lines-$linecounter-1] = fgets($handle);
        
                if($beginning) {
                        break;
                }
       }
       fclose ($handle);
       return rtrim(implode("", array_reverse($text))); 
}


// ----------------------------------------------
// Gets all of the words from all of the arguments passed in
// that appear on the dc_blacklist
function get_blacklisted_words()
{	
	$args = func_get_args();
	$fulltext = implode(" ", $args);
	
	$bad_words = array();
	foreach(get_option('dc_blacklist') as $word)
	{
		if(preg_match("/\b$word\b/i", $fulltext)) $bad_words[] = $word;
	}
	
	if(count($bad_words) > 0)
	{
		return $bad_words;
	}
	return false;
}



// ------------------------------------
function getDirectorySize($path, $this_ext_only=null) 
{ 
	$totalsize = 0; 
	$totalcount = 0; 
	$dircount = 0; 
	if ($handle = opendir ($path)) 
	{ 
		while (false !== ($file = readdir($handle))) 
		{ 
			$nextpath = $path . '/' . $file; 
			$ext = end(explode('.', $nextpath));
			$skip = $ext!=null && $ext!=$this_ext_only;
			if (!$skip && $file != '.' && $file != '..' && !is_link($nextpath)) 
			{ 
				if (is_dir ($nextpath)) 
				{ 
					$dircount++; 
					$result = getDirectorySize($nextpath); 
					$totalsize += $result['size']; 
					$totalcount += $result['count']; 
					$dircount += $result['dircount']; 
				} 
				elseif(is_file($nextpath)) 
				{ 
					$totalsize += filesize ($nextpath); 
					$totalcount++; 
				} 
			} 
		} 
	} 
	closedir ($handle); 
	$total['size'] = $totalsize; 
	$total['count'] = $totalcount; 
	$total['dircount'] = $dircount; 
	return $total; 
} 


// ------------------------------------
function sizeFormat($size) 
{ 
	if($size<1024) 
	{ 
		return $size." bytes"; 
	} 
	else if($size<(1024*1024)) 
	{ 
		$size=round($size/1024,1); 
		return $size." KB"; 
	} 
	else if($size<(1024*1024*1024)) 
	{ 
		$size=round($size/(1024*1024),1); 
		return $size." MB"; 
	} 
	else 
	{ 
		$size=round($size/(1024*1024*1024),1); 
		return $size." GB"; 
	}
}

?>